@@ -112,14 +112,14 @@ def build(input_graph):
         allocate_ipv6 = True
         build_ipv4(anm, infrastructure=True)
 
-    build_ip6(anm)
+    build_ipv6(anm)
     for node in g_phy:
         node.use_ipv4 = allocate_ipv4_infrastructure
         node.use_ipv6 = allocate_ipv6
 
     default_igp = g_in.data.igp or "ospf" 
     non_igp_nodes = [n for n in g_in if not node.igp]
-    g_in.update(non_igp_nodes, igp=default_igp)
+    g_in.update(non_igp_nodes, igp=default_igp) # store igp onto each node
 
     anm.add_overlay("ospf")
     anm.add_overlay("isis")
@@ -145,7 +145,7 @@ def allocate_vrf_roles(g_vrf):
 
     non_ce_nodes = [node for node in g_vrf if node.vrf_role != "CE"]
     for node in non_ce_nodes:
-        phy_neighbors = g_phy.node( node).neighbors()  
+        phy_neighbors = g_phy.node(node).neighbors("is_router")  
         # neighbors from physical graph for connectivity
         phy_neighbors = [neigh for neigh in phy_neighbors if neigh.asn == node.asn]
             # filter to just this asn
@@ -229,26 +229,31 @@ def build_vrf(anm):
         # Set the vrf of the edge to be that of the CE device (either src or dst)
         edge.vrf = edge.src.vrf if edge.src.vrf_role is "CE" else edge.dst.vrf
 
-# Create route-targets
 
-
-def three_tier_ibgp_l1_l3_clusters(rtrs):
+def three_tier_ibgp_corner_cases(rtrs):
     """Calculate edges for iBGP l3 clusters that don't contain a HRR.
     Connects l1 to l3 directly"""
     up_links = []
     down_links = []
+    over_links = []
     for l3_cluster, l3d in ank_utils.groupby("ibgp_l3_cluster", rtrs):
         for l2_cluster, l2d in ank_utils.groupby("ibgp_l2_cluster", l3d):
             l2d = list(l2d)
             if any(r.ibgp_level == 2 for r in l2d):
                 log.debug("Cluster (%s, %s) has l2 devices, not "
                           "adding extra links" % (l3_cluster, l2_cluster))
+            elif all(r.ibgp_level == 1 for r in l2d):
+                # No l2 or l3 routers -> full-mesh of l1 routers
+                over_links += [(s, t) for s in l2d for t in l2d if s != t]
+                log.info("Cluster (%s, %s) has no level 2 or 3 iBGP routers."
+                        "Connecting l1 routers (%s) in full-mesh"
+                        % (l3_cluster, l2_cluster, l2d))
             else:
                 l1_rtrs = [r for r in l2d if r.ibgp_level == 1]
                 l3_rtrs = [r for r in l2d if r.ibgp_level == 3]
                 if not(len(l1_rtrs) and len(l3_rtrs)):
                     break  # no routers to connect
-                log.debug("Cluster (%s, %s) has no level 2 iBGP routers."
+                log.info("Cluster (%s, %s) has no level 2 iBGP routers."
                           "Connecting l1 routers (%s) to l3 routers (%s)"
                           % (l3_cluster, l2_cluster, l1_rtrs, l3_rtrs))
 
@@ -256,9 +261,7 @@ def three_tier_ibgp_l1_l3_clusters(rtrs):
                 up_links += l1_l3_up_links
                 down_links += [(t, s) for (s, t) in l1_l3_up_links]
 
-    return up_links, down_links
-
-
+    return up_links, down_links, over_links
 
 def three_tier_ibgp_edges(routers):
     """Constructs three-tier ibgp"""
@@ -290,9 +293,10 @@ def three_tier_ibgp_edges(routers):
                    if s.ibgp_level == t.ibgp_level == 3]  # l3 peer links
 
 # also check for any clusters which only contain l1 and l3 links
-    l1_l3_up_links, l1_l3_down_links = three_tier_ibgp_l1_l3_clusters(routers)
+    l1_l3_up_links, l1_l3_down_links, l1_l3_over_links = three_tier_ibgp_corner_cases(routers)
     up_links += l1_l3_up_links
     down_links += l1_l3_down_links
+    over_links += l1_l3_over_links
   
     return up_links, down_links, over_links
 
@@ -311,15 +315,30 @@ def build_two_tier_ibgp(routers):
                   and s.ibgp_l3_cluster == t.ibgp_l3_cluster]
     return up_links, down_links, over_links
 
-
 def build_bgp(anm):
     """Build iBGP end eBGP overlays"""
     # eBGP
     g_in = anm['input']
+    g_phy = anm['phy']
     g_bgp = anm.add_overlay("bgp", directed=True)
     g_bgp.add_nodes_from(g_in.nodes("is_router"))
     ebgp_edges = [edge for edge in g_in.edges() if not edge.attr_equal("asn")]
     g_bgp.add_edges_from(ebgp_edges, bidirectional=True, type='ebgp')
+#TODO: why don't we include edge_id here
+
+    ebgp_switches = [n for n in g_in.nodes("is_switch")
+            if not ank_utils.neigh_equal(g_phy, n, "asn")]
+    g_bgp.add_nodes_from(ebgp_switches, retain=['asn'])
+    log.debug("eBGP switches are %s" % ebgp_switches)
+    g_bgp.add_edges_from((e for e in g_in.edges()
+            if e.src in ebgp_switches or e.dst in ebgp_switches), bidirectional=True, type='ebgp')
+    ank_utils.aggregate_nodes(g_bgp, ebgp_switches, retain="edge_id")
+    ebgp_switches = list(g_bgp.nodes("is_switch")) # need to recalculate as may have aggregated
+    log.debug("aggregated eBGP switches are %s" % ebgp_switches)
+    exploded_edges = ank_utils.explode_nodes(g_bgp, ebgp_switches,
+            retain="edge_id")
+    for edge in exploded_edges:
+        edge.multipoint = True
 
 # now iBGP
     ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_level")
@@ -333,6 +352,8 @@ def build_bgp(anm):
         if node.ibgp_level == "None":  # if unicode string from yEd
             node.ibgp_level = 1
 
+#TODO CHECK FOR IBGP NONE
+
         node.ibgp_level = int(node.ibgp_level)  # ensure is numeric
 
         if not node.ibgp_l2_cluster or node.ibgp_l2_cluster == "None":
@@ -349,18 +370,26 @@ def build_bgp(anm):
         ibgp_levels = {int(r.ibgp_level) for r in routers}
         max_level = max(ibgp_levels)
         # all possible edge src/dst pairs
-        all_pairs = [(s, t) for s in routers for t in routers if s != t]
+        ibgp_routers = [r for r in routers if r.ibgp_level > 0]
+        all_pairs = [(s, t) for s in ibgp_routers for t in ibgp_routers if s != t]
         if max_level == 3:
-            up_links, down_links, over_links = three_tier_ibgp_edges(routers)
+            up_links, down_links, over_links = three_tier_ibgp_edges(ibgp_routers)
 
-        if max_level == 2:
-            up_links, down_links, over_links = build_two_tier_ibgp(routers)
+        elif max_level == 2:
+            up_links, down_links, over_links = build_two_tier_ibgp(ibgp_routers)
 
         elif max_level == 1:
             up_links = []
             down_links = []
             over_links = [(s, t) for (s, t) in all_pairs
-                             if s.ibgp_l3_cluster == t.ibgp_l3_cluster]
+                             if s.ibgp_l3_cluster == t.ibgp_l3_cluster
+                             and s.ibgp_l2_cluster == t.ibgp_l2_cluster
+                             ]
+        else:
+            # no iBGP
+            up_links = []
+            down_links = []
+            over_links = []
 
         if max_level > 0:
             g_bgp.add_edges_from(up_links, type='ibgp', direction='up')
@@ -384,15 +413,20 @@ def build_bgp(anm):
         edge.type == 'ebgp' for edge in d.edges())]
     g_bgp.update(ebgp_nodes, ebgp=True)
 
+    for ebgp_edge in g_bgp.edges(type = "ebgp"):
+        for interface in ebgp_edge.interfaces():
+            interface.ebgp = True
+
     for edge in g_bgp.edges(type='ibgp'):
         # TODO: need interface querying/selection. rather than hard-coded ids
         edge.bind_interface(edge.src, 0)
 
-    #for node in g_bgp:
-        #node._interfaces[0]['description'] = "loopback0"
+    #TODO: need to initialise interface zero to be a loopback rather than physical type
+    for node in g_bgp:
+        for interface in node.interfaces():
+            interface.multipoint = any(e.multipoint for e in interface.edges())
 
-
-def build_ip6(anm):
+def build_ipv6(anm):
     """Builds IPv6 graph, using nodes and edges from IPv4 graph"""
     import autonetkit.plugins.ipv6 as ipv6
     # uses the nodes and edges from ipv4
@@ -404,6 +438,16 @@ def build_ip6(anm):
     ipv6.allocate_ips(g_ipv6)
 
 
+    #TODO: replace this with direct allocation to interfaces in ip alloc plugin
+    for node in g_ipv6.nodes("is_l3device"):
+        node.loopback_zero.ip_address = node.loopback
+        for interface in node:
+            edges = list(interface.edges())
+            if len(edges):
+                edge = edges[0] # first (only) edge
+                interface.ip_address = edge.ip #TODO: make this consistent
+                interface.subnet = edge.dst.subnet # from collision domain
+
 def manual_ipv4_allocation(anm):
     """Applies manual IPv4 allocation"""
     import netaddr
@@ -448,7 +492,6 @@ def manual_ipv4_allocation(anm):
     g_ipv4.data.loopback_blocks = loopback_blocks
     g_ipv4.data.infra_blocks = infra_blocks
 
-
 def build_ipv4(anm, infrastructure=True):
     """Builds IPv4 graph"""
     g_ipv4 = anm.add_overlay("ipv4")
@@ -464,8 +507,10 @@ def build_ipv4(anm, infrastructure=True):
 
     edges_to_split = [edge for edge in g_ipv4.edges() if edge.attr_both(
         "is_l3device")]
+    for edge in edges_to_split:
+        edge.split = True # mark as split for use in building nidb
     split_created_nodes = list(
-        ank_utils.split(g_ipv4, edges_to_split, retain='edge_id'))
+        ank_utils.split(g_ipv4, edges_to_split, retain=['edge_id', 'split']))
     for node in split_created_nodes:
         node['graphics'].x = ank_utils.neigh_average(g_ipv4, node, "x",
                                                      g_graphics) + 0.1 # temporary fix for gh-90
@@ -506,6 +551,19 @@ def build_ipv4(anm, infrastructure=True):
         ipv4.allocate_ips(g_ipv4, infrastructure)
         ank_utils.save(g_ipv4)
 
+    autonetkit.update_http(anm)
+    #TODO: replace this with direct allocation to interfaces in ip alloc plugin
+    for node in g_ipv4.nodes("is_l3device"):
+        node.loopback_zero.ip_address = node.loopback
+        for interface in node:
+            edges = list(interface.edges())
+            if len(edges):
+                edge = edges[0] # first (only) edge
+                interface.ip_address = edge.ip_address
+                interface.subnet = edge.dst.subnet # from collision domain
+
+    # TODO: also map loopbacks to loopback interface 0
+    autonetkit.update_http(anm)
 
 def build_phy(anm):
     """Build physical overlay"""
@@ -517,9 +575,16 @@ def build_phy(anm):
         ank_utils.copy_attr_from(g_in, g_phy, "Network")
 
     g_phy.add_edges_from(g_in.edges(type="physical"))
-    g_phy.allocate_interfaces(
-    )  # TODO: make this automatic if adding to the physical graph?
+    # TODO: make this automatic if adding to the physical graph?
+    g_phy.allocate_interfaces() 
 
+    specified_int_names = g_in.data.specified_int_names
+    if specified_int_names:
+        for node in g_phy:
+            for interface in node:
+                edge = interface.edges()[0]
+                directed_edge = anm['input_directed'].edge(edge)
+                interface.name = directed_edge.name
 
 def build_conn(anm):
     """Build connectivity overlay"""
@@ -542,6 +607,8 @@ def build_ospf(anm):
 
     Not-allowed:
     x -> x (x != y != 0)
+
+    #TODO: build check that verifies these rules
     """
     import netaddr
     g_in = anm['input']
@@ -555,8 +622,10 @@ def build_ospf(anm):
 
     ank_utils.aggregate_nodes(g_ospf, g_ospf.nodes("is_switch"),
                               retain="edge_id")
-    ank_utils.explode_nodes(g_ospf, g_ospf.nodes("is_switch"),
+    exploded_edges = ank_utils.explode_nodes(g_ospf, g_ospf.nodes("is_switch"),
                             retain="edge_id")
+    for edge in exploded_edges:
+        edge.multipoint = True
 
     g_ospf.remove_edges_from([link for link in g_ospf.edges(
     ) if link.src.asn != link.dst.asn])  # remove inter-AS links
@@ -635,6 +704,17 @@ def build_ospf(anm):
         if not link.cost:
             link.cost = 1
 
+    # map areas and costs onto interfaces
+    #TODO: later map them directly rather than with edges - this is part of the transition
+    for edge in g_ospf.edges():
+        for interface in edge.interfaces():
+            interface.cost = edge.cost
+            interface.area = edge.area
+            interface.multipoint = edge.multipoint
+
+    for router in g_ospf:
+        router.loopback_zero.area = router.area
+        router.loopback_zero.cost = 0
 
 def ip_to_net_ent_title_ios(ip_addr):
     """ Converts an IP address into an OSI Network Entity Title
@@ -669,8 +749,10 @@ def build_isis(anm):
 # Merge and explode switches
     ank_utils.aggregate_nodes(g_isis, g_isis.nodes("is_switch"),
                               retain="edge_id")
-    ank_utils.explode_nodes(g_isis, g_isis.nodes("is_switch"),
+    exploded_edges = ank_utils.explode_nodes(g_isis, g_isis.nodes("is_switch"),
                             retain="edge_id")
+    for edge in exploded_edges:
+        edge.multipoint = True
 
     g_isis.remove_edges_from(
         [link for link in g_isis.edges() if link.src.asn != link.dst.asn])
@@ -685,6 +767,12 @@ def build_isis(anm):
         # link.hello = 5 # for debugging, TODO: read from graph
 
 
+    for edge in g_isis.edges():
+        for interface in edge.interfaces():
+            interface.metric = edge.metric
+            interface.multipoint = edge.multipoint
+
+
 def update_messaging(anm):
     """Sends ANM to web server"""
     log.debug("Sending anm to messaging")
