@@ -244,10 +244,9 @@ def compile_network(anm):
     nidb.add_nodes_from(
         cd_nodes, retain=['label', 'host'], collision_domain=True)
 # add edges to switches
-    edges_to_add = [edge for edge in g_phy.edges(
-    ) if edge.src.is_switch or edge.dst.is_switch]
-    edges_to_add += [edge for edge in g_ipv4.edges(
-    ) if edge.src.collision_domain or edge.dst.collision_domain]
+    edges_to_add = [edge for edge in g_phy.edges()
+            if edge.src.is_switch or edge.dst.is_switch]
+    edges_to_add += [edge for edge in g_ipv4.edges() if edge.split] # cd edges from split
     nidb.add_edges_from(edges_to_add, retain='edge_id')
 
 # TODO: boundaries is still a work in progress...
@@ -283,6 +282,10 @@ def deploy_network(anm, nidb, input_graph_string):
     deploy_hosts = config.settings['Deploy Hosts']
     for hostname, host_data in deploy_hosts.items():
         for platform, platform_data in host_data.items():
+            if not any(nidb.nodes(host=hostname, platform=platform)):
+                log.debug("No hosts for (host, platform) (%s, %s), skipping deployment"
+                        % (hostname, platform))
+                continue
 
             if not platform_data['deploy']:
                 log.debug("Not deploying to %s on %s" % (platform, hostname))
