@@ -28,10 +28,9 @@ var graph_history = [];
 var ip_allocations = [];
 
 var node_label_id = "id";
-var edge_group_id = "";
+var edge_group_id = ""; //TODO: rename edge_label_id
 var interface_label_id = "";
 
-
 starting_hosts = [];
 
 ws.onmessage = function (evt) {
@@ -86,7 +85,7 @@ ws.onmessage = function (evt) {
         apply_highlight(data['highlight']);
     }
     else {
-        console.log("Received unknown data", data);
+        //console.log("Received unknown data", data);
         //TODO: work out why reaching here if passing the "graph in data" check above
     }
 }
@@ -406,10 +405,97 @@ var edge_stroke_colors = d3.scale.ordinal();
 
 var groupFill = function(d, i) { return fill(i); };
 var edgeStroke = function(d, i) { return fill(d); };
+
+var interface_angle = function(d){
+    //common to interface_x and interface_y
+    s_x = node_x(d.node);
+    s_y = node_y(d.node);
+    t_x = node_x(d.target);
+    t_y = node_y(d.target);
+
+    angle = Math.atan2( (t_x - s_x), (t_y - s_y));
+    return angle;
+}
+
+
+var interface_width = 15;
+var interface_height = 10;
+
+var interface_hypotenuse = (icon_width + icon_height)/2;
+
+var interface_x = function(d) {
+
+    if (jsondata.directed) {
+        return directed_edge_offset_x(d.node, d.target, interface_hypotenuse) - interface_width/2;
+    }
+
+    angle = interface_angle(d);
+    offset_x = interface_hypotenuse * Math.sin(angle);
+    return node_x(d.node) + offset_x - interface_width/2;
+}
+
+var interface_y = function(d) {
+
+    if (jsondata.directed) {
+        return directed_edge_offset_y(d.node, d.target, interface_hypotenuse) - interface_height/2;
+    }
+
+    angle = interface_angle(d);
+    offset_y =interface_hypotenuse * Math.cos(angle);
+    return node_y(d.node) + offset_y - interface_height/2;
+}
+
+
 var groupPath = function(d) {
+
+    if (display_interfaces && overlay_id == "ospf") {
+        if (d.values.length  == 1) {
+            //This shouldn't occur for single ospf interface!
+            node = d.values[0];
+            offset = 2;
+            retval =  "M" ;
+            retval += (interface_x - offset  + icon_offset + x_offset) + "," + (interface_y - offset + icon_offset + y_offset) + "L";
+            retval += (interface_x + offset  + icon_offset + x_offset) + "," + (interface_y - offset + icon_offset + y_offset) + "L";
+            retval += (interface_x - offset  + icon_offset + x_offset) + "," + (interface_y + offset + icon_offset + y_offset) + "L";
+            retval += (interface_x + offset + icon_offset + x_offset) + "," + (interface_y + offset + icon_offset + y_offset);
+            retval += "Z";
+            return retval
+        } else if (d.values.length  == 2) {
+                interface1 = d.values[1];
+                interface2 = d.values[0];
+                //Note: upper_x goes with lower_y due to y ascending down, x ascending right
+                int1_x = interface_x(interface1);
+                int1_y = interface_y(interface1);
+                int2_x = interface_x(interface2);
+                int2_y = interface_y(interface2);
+                upper_x = Math.max(int1_x, int2_x);
+                upper_y = Math.max(int1_y, int2_y);
+                lower_x = Math.min(int1_x, int2_x);
+                lower_y = Math.min(int1_y, int2_y);
+
+                offset = 0.1;
+                retval =  "M" ;
+                retval += (upper_x - offset ) + "," + (upper_y - offset ) + "L";
+                retval += (upper_x + offset ) + "," + (upper_y - offset ) + "L";
+                retval += (lower_x + offset ) + "," + (lower_y + offset ) + "L";
+                retval += (lower_x - offset ) + "," + (lower_y + offset ) ;
+                retval += "Z";
+                return retval;
+        } else {
+            retval = "M" + 
+                d3.geom.hull(d.values.map(function(i) { 
+                    return [interface_x(i), interface_y(i)]; 
+                }))
+                .join("L")
+                + "Z";
+            return retval;
+        }
+        return;
+    }
+
     if (d.values.length  == 1) {
         node = d.values[0];
-        offset = 10;
+        offset = 2;
         retval =  "M" ;
         retval += (node.x - offset  + icon_offset + x_offset) + "," + (node.y - offset + icon_offset + y_offset) + "L";
         retval += (node.x + offset  + icon_offset + x_offset) + "," + (node.y - offset + icon_offset + y_offset) + "L";
@@ -424,6 +510,7 @@ var groupPath = function(d) {
         node1 = d.values[1];
         node2 = d.values[0];
         //Note: upper_x goes with lower_y due to y ascending down, x ascending right
+        ////TODO: do these get over written?
         upper_x = Math.max(node1.x, node2.x);
         upper_y = Math.max(node1.y, node2.y);
         lower_x = Math.min(node1.x, node2.x);
@@ -523,6 +610,7 @@ var interface_info = function(d) {
     int_data = d.node._interfaces[d.interface];
 
     text = "<ul>"; //begin the unordered list
+    text += "<li><b>id:</b> " + d.interface + "</li>"; //add the key/val
     for (attr in int_data) {
         text += "<li><b>" + attr + ":</b> " + int_data[attr] + "</li>"; //add the key/val
     }
@@ -823,8 +911,14 @@ var device_label = function(d) {
 }
 
 var interface_label = function(d) {
+    try {
     int_data = d.node._interfaces[d.interface];
     return int_data[interface_label_id];
+    }
+    catch (err) {
+        //console.log(err);
+        //console.log("error for", d.node.id, d.interface);
+    }
 }
 
 var zoom_fit = function() {
@@ -987,13 +1081,69 @@ function redraw() {
             return _.keys(interface);
         });
     });
-    interface_attributes = _.flatten(interface_attributes); //collapse from hierarchical nested structure
-    interface_attributes = _.uniq(interface_attributes);
-    propagate_interface_label_select(interface_attributes);
+    //collapse from hierarchical nested structure
+    interface_attributes_flattened = _.flatten(interface_attributes);
+    interface_attributes_unique = _.uniq(interface_attributes_flattened);
+    propagate_interface_label_select(interface_attributes_unique);
 
     node_attr_groups = d3.nest().key( node_group_id ).entries(nodes);
     edge_attr_groups = d3.nest().key(function(d) { return d[edge_group_id]; }).entries(jsondata.links);
     //TODO: use edge attr groups for edge colours
+    
+    //If undirected graph, then need two interfaces per edge: one at each end
+    if (display_interfaces) {
+        //Undirected, need to handle for both src and dst
+        interface_data = _.map(jsondata.links, function(link) {
+            interface_data = link._interfaces;
+            src_node = nodes[link.source];
+            dst_node = nodes[link.target];
+            src_int_id = interface_data[src_node.id]; //interface id is indexed by the node id
+            dst_int_id = interface_data[dst_node.id]; //interface id is indexed by the node id
+
+            //Check for null interace ids: some nodes may not have interfaces (eg a collision domain)
+
+            //TODO: if a directed link, only return for source
+            //
+            retval = [];
+            if (src_int_id != null) {
+                retval.push( { 'node': src_node, 'interface':  src_int_id, 'target': dst_node, 'link': link });
+            }
+
+            if (!jsondata.directed && dst_int_id != null) {
+                //undirected, also include data for other interface
+                retval.push( { 'node': dst_node, 'interface':  dst_int_id, 'target': src_node, 'link': link });
+                }
+
+            return retval;
+
+        });
+
+        interface_data = _.flatten(interface_data); //collapse from hierarchical nested structure
+    } else {
+        interface_data = {}; //reset 
+    }
+
+
+    var interface_area = function(d) {
+        interface_id = d.interface;
+        return d.node._interfaces[interface_id]['area'];
+    }
+
+    interface_attr_groups = d3.nest().key( interface_area ).entries(interface_data);
+
+    if (display_interfaces && overlay_id == "ospf") {
+        node_attr_groups = interface_attr_groups;
+    }
+
+    var hull_stroke_width = function() {
+
+        if (display_interfaces && overlay_id == "ospf") {
+            return 40;
+        }
+        return 80;
+
+    }
+
 
     //TODO: make group path change/exit with node data
     groupings = chart.selectAll(".attr_group")
@@ -1004,7 +1154,7 @@ function redraw() {
         .attr("d", groupPath)
         .style("fill", groupFill)
         .style("stroke", groupFill)
-        .style("stroke-width", 80)
+        .style("stroke-width", hull_stroke_width)
         .style("stroke-linejoin", "round")
         .style("opacity", 0.15)
         .on("mouseover", function(d){
@@ -1156,34 +1306,7 @@ function redraw() {
         .remove();
 
 
-    //If undirected graph, then need two interfaces per edge: one at each end
-    if (display_interfaces) {
-        //Undirected, need to handle for both src and dst
-        interface_data = _.map(jsondata.links, function(link) {
-            interface_data = link._interfaces;
-            src_node = nodes[link.source];
-            dst_node = nodes[link.target];
-            src_int_id = interface_data[src_node.id]; //interface id is indexed by the node id
-            dst_int_id = interface_data[dst_node.id]; //interface id is indexed by the node id
 
-            //TODO: if a directed link, only return for source
-            //
-            retval = [];
-            retval.push( { 'node': src_node, 'interface':  src_int_id, 'target': dst_node, 'link': link });
-
-            if (!jsondata.directed) {
-                //undirected, also include data for other interface
-                retval.push( { 'node': dst_node, 'interface':  dst_int_id, 'target': src_node, 'link': link });
-                }
-
-            return retval;
-
-        });
-
-        interface_data = _.flatten(interface_data); //collapse from hierarchical nested structure
-    } else {
-        interface_data = {}; //reset 
-    }
 
     //TODO: handle removing of interfaces
 
@@ -1205,47 +1328,12 @@ function redraw() {
         .duration(4000);
 
 
+    //console.log(interface_data);
 
     interface_icons = chart.selectAll(".interface_icon")
         //.data(interface_data) //TODO: check if need to provide an index
+        //TODO: check if should return tuple of interface, node for uniqueness (esp for switching overlays)
         .data(interface_data, function(d) { return d.interface;})
-
-        var interface_width = 15;
-        var interface_height = 10;
-
-        var interface_angle = function(d){
-            //common to interface_x and interface_y
-            s_x = node_x(d.node);
-            s_y = node_y(d.node);
-            t_x = node_x(d.target);
-            t_y = node_y(d.target);
-
-            angle = Math.atan2( (t_x - s_x), (t_y - s_y));
-            return angle;
-        }
-
-        var interface_hypotenuse = (icon_width + icon_height)/2;
-
-        var interface_x = function(d) {
-
-            if (jsondata.directed) {
-                return directed_edge_offset_x(d.node, d.target, interface_hypotenuse) - interface_width/2;
-            }
-    
-            angle = interface_angle(d);
-            offset_x = interface_hypotenuse * Math.sin(angle);
-            return node_x(d.node) + offset_x - interface_width/2;
-        }
-        var interface_y = function(d) {
-
-            if (jsondata.directed) {
-                return directed_edge_offset_y(d.node, d.target, interface_hypotenuse) - interface_height/2;
-            }
-
-            angle = interface_angle(d);
-            offset_y =interface_hypotenuse * Math.cos(angle);
-            return node_y(d.node) + offset_y - interface_height/2;
-        }
         
         var highlight_interfaces = function(d) {
             interfaces = d3.selectAll(".interface_icon");
@@ -1343,6 +1431,13 @@ function redraw() {
             .attr("dx", 0) // padding-right
             .attr("dy", 0) // vertical-align: middle
             .text(function (d) {
+                if (edge_group_id == "_interfaces") {
+                    retval = "";
+                    for (attr in d[edge_group_id]) {
+                        retval += "(" + attr + ", " + d[edge_group_id][attr] + ")";
+                    }
+                    return retval;
+                }
                 return d[edge_group_id];
             });
 
